<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.trustt.io
 * @since      1.0.0
 *
 * @package    Trustt
 * @subpackage Trustt/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Trustt
 * @subpackage Trustt/admin
 * @author     Dimitri Longo <info@elanathemes.fr>
 */
class Trustt_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $trustt    The ID of this plugin.
	 */
	private $trustt;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version
	 */
	private $version;

	/**
	 * The version of json orders.
	 *
	 * @since    1.1.0
	 * @access   private
	 * @var      string    $version_json_orders
	 */
	private $version_json_orders;

	/**
	 * wp_options name for config tab.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $options_config    The name of option for config.
	 */
	private $options_config;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $trustt
	 * @param      string    $version
	 */
	public function __construct( $trustt, $version, $version_json_orders, $options_config ) {

		$this->trustt              = $trustt;
		$this->version             = $version;
		$this->version_json_orders = $version_json_orders;
		$this->options_config      = $options_config;
		$this->is_admin            = is_admin();
		$this->plugin_basename     = plugin_basename( plugin_dir_path( __DIR__ ) . $this->trustt . '.php' );
		$this->ifdebug();

		add_action( 'rest_api_init', array( $this, 'el_woo_version_check' ) );

		add_action( 'rest_api_init', array( $this, 'restapiinit' ) );

		add_action( 'rest_api_init', array( $this, 'el_rest_api_init_orders' ) );

		// Add an action link pointing to the options page.
		add_filter( 'plugin_action_links_' . $this->plugin_basename, array( $this, 'el_action_links' ) );
		add_filter( 'plugin_row_meta', array( $this, 'el_plugin_row_meta' ), 10, 2 );

	}

	/**
	 * Add pluging row meta links.
	 *
	 * @since    1.0.0
	 */
	public function el_plugin_row_meta( $links, $file ) {
		if ( $this->plugin_basename == $file ) {
			$row_meta = array(
			  'docs'    => '<a href="' . esc_url( 'https://trustt.helpscoutdocs.com/article/96-comment-installer-trustt-reviews-sur-un-site-wordpress-avec-woocommerce' ) . '" target="_blank" aria-label="' . esc_attr__( 'Documentation', 'trustt' ) . '">' . esc_html__( 'Documentation', 'trustt' ) . '</a>',
			  'help'    => '<a href="' . esc_url( 'https://trustt.helpscoutdocs.com' ) . '" target="_blank" aria-label="' . esc_attr__( 'Centre d\'Aide', 'trustt' ) . '">' . esc_html__( 'Centre d\'Aide', 'trustt' ) . '</a>'

			);

			return array_merge( $links, $row_meta );
		}
		return (array) $links;
	}

	/**
	 * Add settings link.
	 *
	 * @since    1.0.0
	 */
	public function el_action_links( $links ) {

		return array_merge(
			array(
				'settings' => '<a href="' . admin_url( 'options-general.php?page=el-trustt-config') . '">' . __( 'Réglages', 'trustt' ) . '</a>'
			),
			$links
		);

	}
	/**
	 * Admin notice if Woocommerce not activated or installed
	 *
	 * @since 1.0.0
	 */
	public function el_admin_notice_activation() {

		if ( !is_plugin_active( 'woocommerce/woocommerce.php') ) {

			$html  = '<div class="error">';
			$html .= '<p>';
			$html .= sprintf( __( 'Trustt Reviews : Attention Woocommerce n\'est pas installé ou activé, rendez-vous sur la page des <strong><a href="%s">Extensions</a></strong>.', 'trustt' ), admin_url( '/plugins.php') );
			$html .= '</p>';
			$html .= '</div><!-- /.error -->';

			echo $html;
		}
	}

	/**
	 * Redirect after plugin activated
	 *
	 * @since 1.0.0
	 */
	public function el_redirect_activate_plugin() {

		$currentScreen = get_current_screen();
		$screen        = array( 'plugins' );

		if ( true == get_option( 'redirect-trust-welcome-page' ) && in_array( $currentScreen->id, $screen ) ) {

			wp_redirect("/wp-admin/admin.php?page=el-trustt-config");
			delete_option( 'redirect-trust-welcome-page' );

		}
	}

	/**
	 * Check woocommerce version
	 *
	 * @since 1.1.0
	 */
	public function el_woo_version_check() {
		$versionwoo = '3.0';
		if ( ! class_exists( 'WooCommerce' ) ) {
			//woocommerce is not activated or installed
			return false;
		}

		if ( version_compare( WC_VERSION, $versionwoo, '>=' ) ) {
			//it's installed, activated, and a high enough version
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Serve a REST API request for orders
	 *
	 * @return Object
	 * @since   1.1.0
	 */
	public function el_rest_api_init_orders() {
		register_rest_route( 'tbr-api', '/orders/', array(
			'methods'  => 'GET',
			'callback' => array( $this,'get_orders_data' ),
			'permission_callback' => '__return_true',
		) );
	}

	/**
	 * get orders json
	 *
 	 * @param array $data Options for the function.
	 * @return Json
	 * @since   1.1.0
	 */
	public function get_orders_data( $data ) {

		global $woocommerce;

		// get api key from url query parameters
		$apikey    = $data->get_param( 'tb_key' );

		// get order startdate from url query parameters
		$startdate = $data->get_param( 'start_date' );

		// get order enddate from url query parameters
		$enddate   = $data->get_param( 'end_date' );

		// get query limit from url query parameters
		$limit     = $data->get_param( 'limit' );

		if ( ! empty ( $startdate ) ) {

			$startdatesss = date_create_from_format(DATE_ATOM, $startdate );
			$startdate = date_format( $startdatesss, 'Y-m-d' );

		} else {
			$startdate = date('Y-m-d', strtotime( 'today' . '- 3 days' ) );
		}

		if ( ! empty ( $enddate ) ) {
			$enddatesss = date_create_from_format(DATE_ATOM, $enddate );
			$enddate = date_format( $enddatesss, 'Y-m-d' );
		} else {
			$enddate = date('Y-m-d', strtotime( 'today' ) );

		}

		if ( ! empty ( $limit ) ) {
			$limit = $data->get_param( 'limit' );
		} else {
			$limit =  500;
		}

		// get selected product id from url query parameters
		$productid      = $data->get_param( 'id' );

		// strings id to array
		$selected_products_ids = explode( ',', $productid );

		if ( ! empty ( $apikey ) ) {
			// Test Api Key
			$testapi        = Trustt_Admin::get_remote( $apikey );

			$tmp            = array_values( $testapi );

			// Get key value
			$token          = array_shift( $tmp );

			// Get key
			$error_or_token = array_key_first( $testapi );

		} else {
			$error_or_token = 'error';
		}

		if ( $error_or_token == 'error' ) {

			// If error display json error message
			$message = [];
			$message['error'] = 'Invalid tokens';

			return $message;

		} else if ( ! $this->el_woo_version_check() ) {
			// If Woo not activated or installed display json error message
			$message = [];
			$message['error'] = 'woocommerce is not activated or installed or wrong version, min required 3.0';

			return $message;

		} else {
			$options = get_option('el-settings-trustt');
			$el_setting_trustt_status_order = isset( $options['el_setting_trustt_status_order'] );
			$el_setting_trustt_status_order = $el_setting_trustt_status_order ? $options['el_setting_trustt_status_order'] : 'wc-completed';

			$query = new WC_Order_Query( array(
				'limit'   => $limit,
				'orderby' => 'date',
				'order'   => 'DESC',
				'return'  => 'ids',
				'date_created'=> $startdate .'...'. $enddate
			) );

			$orders = $query->get_orders();

			// init empty array with root "data"
			$orderdatameta                                              = array( 'meta' => array() );
			$orderdatameta['meta']['success']                           = 'true';
			$orderdatameta['meta']['count']                             = count( $orders);
			$orderdatameta['meta']['plateform']                         = 'woocommerce';
			$orderdatameta['meta']['woo_version']                       = $woocommerce->version;
			$orderdatameta['meta']['wp_version']                        = get_bloginfo( 'version' );
			$orderdatameta['meta']['module_version']                    = $this->version;
			$orderdatameta['meta']['version']                           = $this->version_json_orders;
			$orderdatameta['meta']['test_startdate_atom_query_convert_to_wp_query'] = $startdate;
			$orderdatameta['meta']['test_enddate_atom_query_convert_to_wp_query']   = $enddate;

			$orderdata = array( 'data' => array() );
			// set counter to 0 for array later on
			$n = 0;
			$status = '';
			foreach ( $orders as $order_id ) {
				$order = wc_get_order( $order_id );

				if ( 'wc-' . $order->get_status() == $el_setting_trustt_status_order ) {
					$status = true;
				} else {
					$status = false;
				}

				$user_data       = get_userdata( $order->get_customer_id() );
				$registered_date = date( DATE_ATOM, strtotime( $user_data->user_registered ) );
				$locale = substr( get_user_locale( $order->get_customer_id() ), 0, 2) ;
				$orderdata['data'][$n]['id']                      = $order->get_id();
				$orderdata['data'][$n]['reference']               = $order->get_id();
				$orderdata['data'][$n]['date']                    = date(DATE_ATOM, strtotime($order->order_date));
				$orderdata['data'][$n]['date_update']             = date(DATE_ATOM, strtotime($order->modified_date));
				$orderdata['data'][$n]['id_shop']                 = '';
				$orderdata['data'][$n]['lang']                    = $locale;
				$orderdata['data'][$n]['customer']['id']          = $order->get_customer_id();
				$orderdata['data'][$n]['customer']['email']       = $order->get_billing_email();
				$orderdata['data'][$n]['customer']['firstname']   = $order->get_billing_first_name();
				$orderdata['data'][$n]['customer']['lastname']    = $order->get_billing_last_name();
				$orderdata['data'][$n]['customer']['date_add']    = $registered_date;
				$orderdata['data'][$n]['currency']                = $order->get_currency();
				$orderdata['data'][$n]['total_paid_tax_incl']     = $order->get_total() - 0;
				$orderdata['data'][$n]['total_paid_tax_excl']     = round( ( $order->get_total() - 0 ) - $order->get_total_tax(), 2);
				$orderdata['data'][$n]['total_shipping_tax_incl'] = $order->get_shipping_total() + $order->get_shipping_tax();
				$orderdata['data'][$n]['coupons']                 = $order->get_coupon_codes();
				$orderdata['data'][$n]['state']                   = $order->get_status();
				$orderdata['data'][$n]['send_email']              = $status;

				// init posts counter
				$i = 0;
				foreach( $order->get_items() as $item_id => $item ) {
					$orderdata['data'][$n]['products'][$i]['id_product']              = $item->get_product_id();
					$orderdata['data'][$n]['products'][$i]['id_attribute']            = '';
					$orderdata['data'][$n]['products'][$i]['quantity']                = $item->get_quantity();
					$orderdata['data'][$n]['products'][$i]['unit_price_incl']         = round( ( ( $item->get_total() + $item->get_total_tax() ) / $item->get_quantity() ), 2);
					$orderdata['data'][$n]['products'][$i]['unit_price_catalog_incl'] = round( ( ( $item->get_total() + $item->get_total_tax() ) / $item->get_quantity() ), 2);
					$orderdata['data'][$n]['products'][$i]['label']                   = $item->get_name();

					$i++;
				}

				// increment counter for array
				$n++;
			}
			$result = array_merge( $orderdatameta, $orderdata );

			if ( empty( $orderdata ) && ! empty ( $productid ) ) {
				// If error display json error message
				$message = [];
				$message['error'] = 'no product found';

				return $message;
			} elseif ( empty( $orderdata ) ) {
				return $result;
			} else {
				return $result;
			}
		}

	}

	/**
	 * Gets the response from the Trustt API and returns the body.
	 *
	 * @param string $apikey The API Key to check or.
	 * @param array  $host The authorized host
	 *
	 * @return array The successful response or the error details.
	 */
	public function restapiinit() {
		register_rest_route( 'tbr-api', '/products/', array(
			'methods'  => 'GET',
			'callback' => array($this,'get_product_data'),
			'permission_callback' => '__return_true',
		) );
	}

	public function get_product_data( $data ) {
		// get api key from url query parameters
		$apikey         = $data->get_param( 'tb_key' );

		// get selected product id from url query parameters
		$productid      = $data->get_param( 'id' );

		// strings id to array
		$selected_products_ids = explode( ',', $productid );

		if ( ! empty ( $apikey ) ) {
			// Test Api Key
			$testapi        = Trustt_Admin::get_remote( $apikey );

			$tmp            = array_values( $testapi );

			// Get key value
			$token          = array_shift( $tmp );

			// Get key
			$error_or_token = array_key_first( $testapi );

		} else {
			$error_or_token = 'error';
		}
		if ( $error_or_token == 'error' )  {

			// If error display json error message
			$message = [];
			$message['error'] = 'Invalid tokens';

			return $message;

		} else {

			if ( ! empty ($productid ) ) {
				$args = [
					'numberposts' => -1,
					'post_type'   => 'product',
					'post_status' => 'publish',
					'post__in'    => $selected_products_ids
				];
			} else {
				$args = [
					'numberposts' => -1,
					'post_type'   => 'product',
					'post_status' => 'publish',
				];
			}

			$posts       = get_posts( $args );
			$productdata = [];
			$i           = 0;
			foreach( $posts as $post ) {
				$product = wc_get_product( $post->ID );
				$cover = get_the_post_thumbnail_url( $post->ID ) ? get_the_post_thumbnail_url( $post->ID ) : '';
				$productdata[$i]['product']['id']                = $post->ID;
				$productdata[$i]['product']['name']              = $post->post_title;
				$productdata[$i]['product']['description_short'] = $post->post_excerpt;
				$productdata[$i]['pricePublic']                  = $product->get_price();
				$productdata[$i]['url']                          = get_permalink( $post->ID );
				$productdata[$i]['cover']                        = $cover;
				$productdata[$i]['images']['id_image']           = $post->post_title;
				$productdata[$i]['images']['cover']              = 1;
				$productdata[$i]['images']['legend']             = get_the_post_thumbnail_caption( $post->ID );
				$productdata[$i]['images']['position']           = 1;
				$productdata[$i]['images']['url']                = $cover;
				$i++;
			}
			if ( empty( $productdata ) && ! empty ( $productid ) ) {
				// If error display json error message
				$message = [];
				$message['error'] = 'no product found';

				return $message;
			} elseif ( empty( $productdata ) ) {
				return $productdata;
			} else {
				return $productdata;
			}
		}

	}

	/**
	 * Gets the response from the Trustt API and returns the body.
	 *
	 * @param string $apikey The API Key to check or.
	 * @param array  $host The authorized host
	 *
	 * @return array The successful response or the error details.
	 */
	public static function get_remote( $apikey ) {

		$options                       = get_option( 'el-settings-trustt' );
		$el_setting_section_trustt_ssl = isset( $options['el_setting_section_trustt_ssl'] );
		$el_setting_section_trustt_ssl = $el_setting_section_trustt_ssl ? $options['el_setting_section_trustt_ssl'] : 0;

		$defaults = [
			'api_key'   => $apikey,
			'host'   	=> get_site_url(),
		];

		if ( $el_setting_section_trustt_ssl ) {
			$args_for_get = array(
				'sslverify' => true,
			);
		} else {
			$args_for_get = array(
				'sslverify' => false,
			);
		}

		$trustt_endpoint = 'https://' . TRUSTT_API_ENDPOINT . '/api/check';
		$parameters      = array_merge( $defaults );
		$url             = add_query_arg( $parameters, $trustt_endpoint );
		$response        = wp_remote_get( $url, $args_for_get );

		$process_response = self::process_response( $response );
		return $process_response;
	}

	/**
	 * Processes the given Trustt response.
	 *
	 * @param array|WP_Error $response The response or WP_Error to process.
	 *
	 * @return array The response body or the error detaiils on failure.
	 */
	public static function process_response( $response ) {
		// Most of the potential errors are WP_Error(s).
		if ( is_wp_error( $response ) ) {
			return [
				'is_error'       => true,
				'raw_error_code' => '',
				// WP_Error codes aren't that helpful for users, let's display them in a less prominent way.
				'wp_error_code'  => '(' . $response->get_error_code() . ')',
				'message'        => $response->get_error_message(),
			];
		}

		// When the request is successful, the response code will be 200.
		$response_body = wp_remote_retrieve_body( $response );

		$details = json_decode( $response_body, true );

		$tmp = array_values($details);

		$token = array_shift( $tmp );

		$error_or_token = array_key_first($details);
		$output = get_option('el-settings-trustt');

		if ( $error_or_token == 'error' )  {
			update_option( 'trustt_api_check', 0 );
			update_option( 'trustt_api_token',  '' );
		} elseif ( $error_or_token == 'token' ) {
			update_option( 'trustt_api_check', 1 );
			update_option( 'trustt_api_token',  $token );
		} else {
			update_option( 'trustt_api_check', 0 );
			update_option( 'trustt_api_token',  '' );
		}

		return json_decode( $response_body, true );
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		$min           = $this->ifdebug();
		$currentScreen = get_current_screen();
		$screen        = array( 'trustt-reviews_page_el-trustt-ecommerce', 'trustt-reviews_page_el-trustt-general', 'toplevel_page_el-trustt-config', 'trustt-reviews_page_el-trustt-orders' );
		if ( in_array( $currentScreen->id, $screen ) ) {
			wp_enqueue_style( $this->trustt, plugin_dir_url( __FILE__ ) . 'css/trustt-admin'.$min.'css', array(), $this->version, 'all' );
			wp_enqueue_style( 'uikit', plugin_dir_url( __FILE__ ) . 'css/uikit'.$min.'css', array(), $this->version, 'all' );
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		$min           = $this->ifdebug();
		$currentScreen = get_current_screen();
		$screen        = array( 'trustt-reviews_page_el-trustt-ecommerce', 'trustt-reviews_page_el-trustt-general','toplevel_page_el-trustt-config', 'trustt-reviews_page_el-trustt-orders' );
		if ( in_array( $currentScreen->id, $screen ) ) {
			wp_enqueue_script( $this->trustt, plugin_dir_url( __FILE__ ) . 'js/trustt-admin'.$min.'js', array( 'jquery' ), $this->version, false );
			wp_enqueue_script( 'uikit-js', plugin_dir_url( __FILE__ ) . 'js/uikit'.$min.'js', array(), $this->version, false );
			wp_enqueue_script( 'uikit-icons-js', plugin_dir_url( __FILE__ ) . 'js/uikit-icons'.$min.'js', array(), $this->version, false );
		}
	}

	/**
	 *  Check if wp_debug is TRUE
	 *
	 * @since    1.0.0
	 */
	private function ifdebug() {

		if ( WP_DEBUG ) {
				$min ='.';
			} else {
				$min ='.min.';
			}
			return $min;
	}

	/**
	 *  Check if apicheck is TRUE
	 *
	 * @since    1.0.0
	 */
	public static function trusttapicheck() {

		$apicheck = get_option('trustt_api_check');

		if ( $apicheck == 1 ) {
			return true;
		} else {
			return false;
		}
	}

}
