<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.trustt.io/
 * @since      1.0.0
 *
 * @package    Trustt
 * @subpackage Trustt/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Trustt
 * @subpackage Trustt/includes
 * @author     Your Name <email@example.com>
 */

class Trustt {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Trustt_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $trustt    The string used to uniquely identify this plugin.
	 */
	protected $trustt;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   public
	 * @var      string    $version    The current version of the plugin.
	 */
    public $version;

	/**
	 * The version of json orders.
	 *
	 * @since    1.1.0
	 * @access   private
	 * @var      string    $version_json_orders
	 */
	protected $version_json_orders;

	/**
	 * The options of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $options_config
	 */
	protected $options_config;


    public $plugin_slug;
    public $cache_key;
    public $cache_allowed;

    /**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'TRUSTT_VERSION' ) ) {
			$this->version = TRUSTT_VERSION;
		} else {
			$this->version = '1.0.0';
		}

		if ( defined( 'TRUSTT_VERSION_JSON_ORDERS' ) ) {
			$this->version_json_orders = TRUSTT_VERSION_JSON_ORDERS;
		} else {
			$this->version_json_orders = '1.0.0';
		}
		$this->trustt = 'trustt';
		$this->options_config = 'el-settings-trustt';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

        $this->plugin_slug = 'trustt-reviews-woocommerce';
        $this->cache_key = 'trustt_reviews_custom_upd';
        $this->cache_allowed = true;

        add_filter( 'plugins_api', array( $this, 'info' ), 20, 3 );
        add_filter( 'site_transient_update_plugins', array( $this, 'update' ) );
        add_action( 'upgrader_process_complete', array( $this, 'purge' ), 10, 2 );

    }

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Trustt_Loader. Orchestrates the hooks of the plugin.
	 * - Trustt_i18n. Defines internationalization functionality.
	 * - Trustt_Admin. Defines all hooks for the admin area.
	 * - Trustt_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-trustt-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-trustt-i18n.php';

		/**
		 * The class responsible for Helper
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-trustt-helper.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-trustt-admin.php';

		/**
		 * The class responsible for defining all settings fields.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-trustt-admin-config.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-trustt-public.php';

		/**
		 * The class responsible for orchestrating the widgets. NOT NEEDED NOW
		 *
		 */
		//require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-trustt-widget.php';

		$this->loader = new Trustt_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Trustt_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Trustt_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Trustt_Admin( $this->get_trustt(), $this->get_version(), $this->get_version_json_orders(), $this->get_options_config() );

		$plugin_admin_config = new Trustt_Admin_Config( $this->get_trustt(), $this->get_version(), $this->get_version_json_orders(), $this->get_options_config() );

		//Redirect on plugin activated
		$this->loader->add_action( 'current_screen', $plugin_admin, 'el_redirect_activate_plugin' );

		//Hooks Admin notices to add if woocommerce activated or installed
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'el_admin_notice_activation' );

		// Hooks into admin_menu hook to add custom page.
		$this->loader->add_action( 'admin_menu', $plugin_admin_config, 'admin_menus' );

		// Init fields for config tab.
		$this->loader->add_action( 'admin_init', $plugin_admin_config, 'el_settings_init' );

		// Buffer the output when reset to defaults.
		$this->loader->add_action( 'admin_init', $plugin_admin_config, 'el_do_output_buffer' );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		// NOT NEEDED NOW V1.0.0
		//$widget = new Trustt_My_Widgets( $this->get_trustt(), $this->get_version(), $this->get_options_config()  );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Trustt\Trustt_Public( $this->get_trustt(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_trustt() {
		return $this->trustt;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Trustt_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	/**
	 * Retrieve the version of json orders.
	 *
	 * @since     1.1.0
	 * @return    string    The version of json orders
	 */
	public function get_version_json_orders() {
		return $this->version_json_orders;
	}

	/**
	 * Retrieve Options config of the plugin.
	 *
	 * @since     1.0.0
	 * @return    array
	 */

	public function get_options_config() {
		return $this->options_config;
	}

    /**
     * ---- PLUGIN UPDATE ----
     */

    public function request(){

        $remote = get_transient( $this->cache_key );

        if( false === $remote || ! $this->cache_allowed ) {

            $remote = wp_remote_get(
                'https://pro.trustt.io/static/downloads/trustt-reviews-woocommerce.json',
                array(
                    'timeout' => 10,
                    'headers' => array(
                        'Accept' => 'application/json'
                    )
                )
            );

            if(
                is_wp_error( $remote )
                || 200 !== wp_remote_retrieve_response_code( $remote )
                || empty( wp_remote_retrieve_body( $remote ) )
            ) {
                return false;
            }

            set_transient( $this->cache_key, $remote, DAY_IN_SECONDS );

        }

        $remote = json_decode( wp_remote_retrieve_body( $remote ) );

        return $remote;

    }


    function info( $res, $action, $args ) {

        // do nothing if you're not getting plugin information right now
        if( 'plugin_information' !== $action ) {
            return $res;
        }

        // do nothing if it is not our plugin
        if( $this->plugin_slug !== $args->slug ) {
            return $res;
        }

        // get updates
        $remote = $this->request();

        if( ! $remote ) {
            return $res;
        }

        $res = new stdClass();

        $res->name = $remote->name;
        $res->slug = $remote->slug;
        $res->version = $remote->version;
        $res->tested = $remote->tested;
        $res->requires = $remote->requires;
        $res->author = $remote->author;
        $res->author_profile = $remote->author_profile;
        $res->download_link = $remote->download_url;
        $res->trunk = $remote->download_url;
        $res->requires_php = $remote->requires_php;
        $res->last_updated = $remote->last_updated;

        $res->sections = array(
            'description' => $remote->sections->description,
            'installation' => $remote->sections->installation,
            'changelog' => $remote->sections->changelog
        );

        if( ! empty( $remote->banners ) ) {
            $res->banners = array(
                'low' => $remote->banners->low,
                'high' => $remote->banners->high
            );
        }

        return $res;

    }

    public function update( $transient ) {

        if ( empty($transient->checked ) ) {
            return $transient;
        }

        $remote = $this->request();

        if(
            $remote
            && version_compare( $this->version, $remote->version, '<' )
            && version_compare( $remote->requires, get_bloginfo( 'version' ), '<=' )
            && version_compare( $remote->requires_php, PHP_VERSION, '<' )
        ) {
            $res = new stdClass();
            $res->slug = $this->plugin_slug;
            $res->plugin = 'trustt-reviews-woocommerce/trustt.php';
            $res->new_version = $remote->version;
            $res->tested = $remote->tested;
            $res->package = $remote->download_url;

            $transient->response[ $res->plugin ] = $res;

        }

        return $transient;

    }

    public function purge($upgrader, $options ){

        if (
            $this->cache_allowed
            && 'update' === $options['action']
            && 'plugin' === $options[ 'type' ]
        ) {
            // just clean the cache when new plugin version is installed
            delete_transient( $this->cache_key );
        }

    }

}
